"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useFund = void 0;
const objects_1 = require("@hypernetlabs/objects");
const contexts_1 = require("../contexts");
const objects_2 = require("../interfaces/objects");
const ethers_1 = require("ethers");
const react_1 = require("react");
const react_alert_1 = require("react-alert");
const constants_1 = require("../constants");
var EActionTypes;
(function (EActionTypes) {
    EActionTypes["FETCHED"] = "FETCHED";
    EActionTypes["TOKEN_SELECTED"] = "TOKEN_SELECTED";
    EActionTypes["SUCCESS"] = "SUCCESS";
    EActionTypes["ERROR"] = "ERROR";
    EActionTypes["AMOUNT_CHANGED"] = "AMOUNT_CHANGED";
})(EActionTypes || (EActionTypes = {}));
function useFund() {
    const { coreProxy } = contexts_1.useStoreContext();
    const { setLoading } = contexts_1.useLayoutContext();
    const alert = react_alert_1.useAlert();
    const initialState = {
        error: null,
        tokenSelectorOptions: [],
        selectedPaymentToken: undefined,
        amount: "1",
    };
    const [state, dispatch] = react_1.useReducer((state, action) => {
        switch (action.type) {
            case EActionTypes.FETCHED:
                return Object.assign(Object.assign({}, state), { error: false, tokenSelectorOptions: action.payload });
            case EActionTypes.TOKEN_SELECTED:
                return Object.assign(Object.assign({}, state), { error: false, selectedPaymentToken: action.payload });
            case EActionTypes.AMOUNT_CHANGED:
                return Object.assign(Object.assign({}, state), { error: false, amount: action.payload });
            case EActionTypes.ERROR:
                return Object.assign(Object.assign({}, state), { error: true });
            case EActionTypes.SUCCESS:
                return Object.assign(Object.assign({}, state), { error: false });
            default:
                return Object.assign({}, state);
        }
    }, initialState);
    react_1.useEffect(() => {
        let cancelRequest = false;
        const fetchData = () => __awaiter(this, void 0, void 0, function* () {
            setLoading(true);
            try {
                if (cancelRequest)
                    return;
                // get data from coreProxy
                coreProxy === null || coreProxy === void 0 ? void 0 : coreProxy.getBalances().map((balance) => {
                    // prepare balances
                    setLoading(false);
                    dispatch({
                        type: EActionTypes.FETCHED,
                        payload: prepareTokenSelector(balance),
                    });
                });
            }
            catch (error) {
                setLoading(false);
                if (cancelRequest)
                    return;
                dispatch({ type: EActionTypes.ERROR });
            }
        });
        fetchData();
        coreProxy === null || coreProxy === void 0 ? void 0 : coreProxy.onBalancesChanged.subscribe({
            next: (balance) => {
                if (cancelRequest)
                    return;
                dispatch({
                    type: EActionTypes.FETCHED,
                    payload: prepareTokenSelector(balance),
                });
            },
        });
        return function cleanup() {
            cancelRequest = true;
        };
    }, []);
    const prepareTokenSelector = (balance) => {
        // TODO: this should be pulled from metamask available balances not the channel balances
        //if (balance.assets.length) {
        if (false) {
            return balance.assets.reduce((acc, assetBalance) => {
                const tokenName = assetBalance.assetAddress === constants_1.ETHER_HEX_ADDRESS
                    ? "ETH"
                    : "HypernetToken";
                acc.push(new objects_2.PaymentTokenOptionViewModel(tokenName, assetBalance.assetAddress));
                return acc;
            }, new Array());
        }
        else {
            return [
                new objects_2.PaymentTokenOptionViewModel("ETH", objects_1.EthereumAddress("0x0000000000000000000000000000000000000000")),
                new objects_2.PaymentTokenOptionViewModel("HyperToken", objects_1.EthereumAddress("0x9FBDa871d559710256a2502A2517b794B482Db40")),
            ];
        }
    };
    const setSelectedPaymentToken = (selectedOption) => {
        dispatch({
            type: EActionTypes.TOKEN_SELECTED,
            payload: selectedOption,
        });
    };
    const setAmount = (enteredAmount) => {
        dispatch({
            type: EActionTypes.AMOUNT_CHANGED,
            payload: enteredAmount,
        });
    };
    const depositFunds = () => {
        var _a, _b;
        if (!((_a = state.selectedPaymentToken) === null || _a === void 0 ? void 0 : _a.address)) {
            dispatch({
                type: EActionTypes.ERROR,
            });
        }
        setLoading(true);
        coreProxy
            .depositFunds((_b = state.selectedPaymentToken) === null || _b === void 0 ? void 0 : _b.address, ethers_1.ethers.utils.parseEther("1"))
            .match((balances) => {
            alert.success("Your fund deposit has succeeded!");
            setLoading(false);
            dispatch({
                type: EActionTypes.SUCCESS,
            });
        }, (err) => {
            alert.error(err.message || "Your fund deposit has failed");
            setLoading(false);
            dispatch({
                type: EActionTypes.ERROR,
            });
        });
    };
    const mintTokens = () => {
        setLoading(true);
        coreProxy.mintTestToken(ethers_1.ethers.utils.parseEther(state.amount || "1")).match(() => {
            alert.success("mint tokens has succeeded");
            setLoading(false);
            dispatch({
                type: EActionTypes.SUCCESS,
                payload: "mint tokens has succeeded",
            });
        }, (err) => {
            alert.error(err.message || "mint tokens has failed");
            setLoading(false);
            dispatch({
                type: EActionTypes.ERROR,
                payload: err.message || "mint tokens has failed",
            });
        });
    };
    return Object.assign(Object.assign({}, state), { setSelectedPaymentToken,
        depositFunds,
        mintTokens,
        setAmount });
}
exports.useFund = useFund;
//# sourceMappingURL=useFund.js.map